﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Resources;
using System.Windows.Media.Imaging;
using Microsoft.Phone;
using System.Collections.ObjectModel;
using Microsoft.Xna.Framework.Media;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Data;
using System.Threading;
using System.Diagnostics;
using System.Threading.Tasks;

class DCT-Test
{
    // * Spremenljivke *

    #region * Spremenljivke *

    #region * razredne spremenljivke - public *
    public WriteableBitmap slika; // objekt slika
    public WriteableBitmap kompresija; // objekt slike, kjer je shranjena nova slika (po izvedbi IDCT)
    public int faktorStiskanja; // faktor stiskanja oziroma stopnja stiskanja
    #endregion

    #region * razredne spremenljivke - private *
    #region * int *
    private int numVrstica = 0; // predstavlja število vrstic (os X)
    private int numStolpec = 0; // predstavlja število stolpcev (os Y)
    private int numNicel = 0;
    private int tmpVelikost = 0;
    private int seq = 0;
    //private int[] sekvenca = new int[64];
    #endregion

    #region * spremenljivke tipa const -> konstante *
    private const int enBajt = 8; // da lahko, v vsaki for zanki uporabimo kar spremenljivko
    private const double ulomek = 0.25; // konstanta -> 1/4 = 0.25 in predstavlja v izračunu ulomek
    private const int N = 7; // konstanta za cik-cak
    private const int OVERFLOW = 2033; // konstanta za cik-cak
    #endregion

    #region * matrike - tip double *
    private double[,] cos = new double[8, 8];
    #endregion

    #region * matrike - tip int *
    private int[,] tmpRed;
    private int[,] tmpGreen;
    private int[,] tmpBlue;
    #endregion

    #region * LIST - tip int *
    private List<int[,]> matrikaRGB = new List<int[,]>(); // izračunane vrednosti pripravljene za IDCT
    private List<int[,]> matrikaFDCT = new List<int[,]>(); // napolnjena z RGB matrikami, prebrane vrednosti slike
    private List<int[,]> matrikaIDCT = new List<int[,]>(); // napolnjena z RGB matrikami, pripravljena za izris slike
    private List<int> cikCak = new List<int>(); // shranjene vrednosti (po izvedbi cik-cak)
    #endregion

    #region * LIST - tip double *
    List<double> shranjeniCas = new List<double>();
    #endregion

    #region * LIST - tip bool *
    private List<bool> datoteka = new List<bool>();
    private List<bool> kodirniNiz = new List<bool>();
    #endregion

    #region * spremenljivke - timer *
    private Stopwatch merilnik = new Stopwatch();
    private Stopwatch izvajanje = new Stopwatch();
    #endregion

    #region * spremenljivke tipa "COLOR" *
    private Color barva;
    #endregion
    #endregion

    // * Pomožne funkcije, ki se izvedejo čisto na začetku *

    #region * Izračun tabele - cosinus *
    private void IzracunCosinus()
    {
        for (int x = 0; x < enBajt; x++)
        {
            for (int y = 0; y < enBajt; y++)
                cos[x, y] = Math.Cos(((2 * x + 1) * y * Math.PI) / 16);
        }
    }
    #endregion

    #region * Vrednost C *
    private double C(int vrednost)
    {
        return (vrednost > 0) ? 1 : (1 / Math.Sqrt(2));
    }
    #endregion

    // * Pomožne funkcije za branje in risanje slike *

    #region Branje slike (RGB)
    private void BranjeSlike()
    {
        for (int i = 0; i < (this.numVrstica * enBajt); i++)
        {
            for (int j = 0; j < (this.numStolpec * enBajt); j++)
            {
                if ((i >= this.slika.Width) || (j >= this.slika.Height))
                {
                    this.tmpRed[i, j] = 0;
                    this.tmpGreen[i, j] = 0;
                    this.tmpBlue[i, j] = 0;
                }
                else
                {
                    this.barva = this.slika.GetPixel(i, j);

                    this.tmpRed[i, j] = this.barva.R - 128;
                    this.tmpGreen[i, j] = this.barva.G - 128;
                    this.tmpBlue[i, j] = this.barva.B - 128;
                }
            }
        }

        this.matrikaFDCT.Add(tmpRed);
        this.matrikaFDCT.Add(tmpGreen);
        this.matrikaFDCT.Add(tmpBlue);
    }
    #endregion

    #region Izris nove slike
    private void IzrisNoveSlike()
    {
        for (int i = 0; i < (this.numVrstica * enBajt); i++)
        {
            for (int j = 0; j < (this.numStolpec * enBajt); j++)
            {
                if ((i < this.slika.Width) && (j < this.slika.Height))
                    this.kompresija.SetPixel(i, j, Color.FromArgb(this.matrikaIDCT[0][i, j], this.matrikaIDCT[1][i, j], this.matrikaIDCT[2][i, j]));
            }
        }
    }
    #endregion

    // * Konstruktor *

    #region * Konstruktor *
    public DCT-Test(Bitmap slika, int faktorStiskanja, bool izbiraPostopka, string potDoDatoteke)
    {
        // "inicializacija" razrednih spremenljivk "public" (prireditev)
        this.slika = new Bitmap(slika);
        this.kompresija = new Bitmap(slika);
        this.faktorStiskanja = faktorStiskanja;
        this.izbiraPostopka = izbiraPostopka;
        this.potDoDatoteke = potDoDatoteke;

        // delitev z 8, ena matrika je 8x8 :) in preverimo, če je velikost slike, deljiva z 8, če ni prištejemo, da dopolnimo, da ne bomo odrezali slike

        #region * vrstice in stolpci *
        this.numVrstica = this.slika.Width / 8;
        this.numStolpec = this.slika.Height / 8;

        if (this.slika.Width % 8 != 0)
            this.numVrstica++;
        if (this.slika.Height % 8 != 0)
            this.numStolpec++;
        #endregion

        #region * inicializacija, določenih matrik, potrebnih za delo (na začetku počistimo, za vsak slučaj) *
        this.matrikaFDCT.Clear();
        this.matrikaRGB.Clear();
        this.matrikaIDCT.Clear();
        this.shranjeniCas.Clear();

        this.tmpRed = new int[this.numVrstica * enBajt, this.numStolpec * enBajt];
        this.tmpGreen = new int[this.numVrstica * enBajt, this.numStolpec * enBajt];
        this.tmpBlue = new int[this.numVrstica * enBajt, this.numStolpec * enBajt];

        this.matrikaRGB.Add(new int[this.numVrstica * enBajt, this.numStolpec * enBajt]);
        this.matrikaRGB.Add(new int[this.numVrstica * enBajt, this.numStolpec * enBajt]);
        this.matrikaRGB.Add(new int[this.numVrstica * enBajt, this.numStolpec * enBajt]);

        this.matrikaIDCT.Add(new int[this.numVrstica * enBajt, this.numStolpec * enBajt]);
        this.matrikaIDCT.Add(new int[this.numVrstica * enBajt, this.numStolpec * enBajt]);
        this.matrikaIDCT.Add(new int[this.numVrstica * enBajt, this.numStolpec * enBajt]);
        #endregion

        // izračun tabele cosinus
        this.IzracunCosinus();

        // preverimo, kateri izbor postopka je izbran

        #region * izbira postopka *
        if (!this.izbiraPostopka)
            IzvediPrvoStopnjo(); // uporabnik je izbral samo FDCT in IDCT
        else if (this.izbiraPostopka)
            IzvediDrugoStopnjo(); // uporabnik je izbral, da želi shraniti sliko tudi v datoteko
        #endregion
    }
    #endregion

    #region * Izbira postopka *
    #region * prva stopnja - FDCT in IDCT *
    private void IzvediPrvoStopnjo()
    {
        this.izvajanje.Start();

        #region Branje slike -> RGB
        this.merilnik.Start();
        this.BranjeSlike();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 0
        #endregion

        #region Algoritem FDCT
        this.merilnik.Restart();
        this.AlgoritemFDCT();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 1
        #endregion

        #region Algoritem IDCT
        this.merilnik.Restart();
        this.AlgoritemIDCT();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 2
        #endregion

        #region Izris slike -> R G B
        this.merilnik.Restart();
        this.IzrisNoveSlike();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 3
        #endregion

        this.izvajanje.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.izvajanje.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 4

        IzpisPrvaStopnja();
    }
    #endregion

    #region * druga stopnja - FDCT, kodiranje, shranjevanje, branje, dekodiranje, IDCT *
    private void IzvediDrugoStopnjo()
    {
        this.izvajanje.Start();

        #region Branje slike -> RGB
        this.merilnik.Start();
        this.BranjeSlike();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 0
        #endregion

        #region Algoritem FDCT
        this.merilnik.Restart();
        this.AlgoritemFDCT();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 1
        #endregion

        #region Cik-Cak
        this.merilnik.Restart();
        this.CikCak();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 2
        #endregion

        #region Kodiranje
        if (this.faktorStiskanja <= 13)
        {
            this.merilnik.Restart();
            this.KodiranjeDo14();
            this.merilnik.Stop();
            this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 3
        }
        else if (this.faktorStiskanja == 14)
        {
            this.merilnik.Restart();
            this.Kodiranje14();
            this.merilnik.Stop();
            this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 3
        }
        else
        {
            this.merilnik.Restart();
            this.Kodiranje15();
            this.merilnik.Stop();
            this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 3
        }
        #endregion

        #region Shranjevanje
        this.merilnik.Restart();
        this.Shranjevanje();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 4
        #endregion


        #region Branje
        this.merilnik.Restart();
        this.Branje();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 5
        #endregion

        #region Dekodiranje
        if (this.faktorStiskanja <= 13)
        {
            this.merilnik.Restart();
            this.DekodiranjeDo14();
            this.merilnik.Stop();
            this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 6
        }
        else if (this.faktorStiskanja == 14)
        {
            this.merilnik.Restart();
            this.Dekodiranje14();
            this.merilnik.Stop();
            this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 6
        }
        else
        {
            this.merilnik.Restart();
            this.Dekodiranje15();
            this.merilnik.Stop();
            this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 6
        }
        #endregion

        #region RE-Cik-Cak
        this.merilnik.Restart();
        this.ReCikCak();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 7
        #endregion

        #region Algoritem IDCT
        this.merilnik.Restart();
        this.AlgoritemIDCT();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 8
        #endregion

        #region Izris slike -> R G B
        this.merilnik.Restart();
        this.IzrisNoveSlike();
        this.merilnik.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.merilnik.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 9
        #endregion

        this.izvajanje.Stop();
        this.shranjeniCas.Add(Math.Round(((double)this.izvajanje.ElapsedTicks / (double)Stopwatch.Frequency), 3)); // 10

        IzpisDrugaStopnja();
    }
    #endregion
    #endregion

    // * FDCT -> Cik-Cak -> Kodiranje -> Shranjevanje *

    #region Algoritem FDCT
    private void AlgoritemFDCT()
    {
        Parallel.For(0, 3, delegate(int numMatrika)
        {
            for (int a = 0; a < this.numVrstica; a++)
            {
                for (int b = 0; b < this.numStolpec; b++)
                {
                    for (int x = 0; x < enBajt; x++)
                    {
                        for (int y = 0; y < enBajt; y++)
                        {
                            if (((7 - x) + (7 - y)) < this.faktorStiskanja)
                                this.matrikaRGB[numMatrika][x + (a * enBajt), y + (b * enBajt)] = 0;
                            else
                                this.matrikaRGB[numMatrika][x + (a * enBajt), y + (b * enBajt)] = this.FDCT(x, y, a, b, this.matrikaFDCT[numMatrika]);
                        }
                    }
                }
            }
        });
    }

    private int FDCT(int u, int v, int numX, int numY, int[,] barva)
    {
        numY = numY * enBajt;
        numX = numX * enBajt;

        double vrednost = 0;

        for (int y = 0; y < enBajt; y++)
        {
            for (int x = 0; x < enBajt; x++)
                vrednost = vrednost + ((double)((barva[y + numX, x + numY])) * cos[x, u] * cos[y, v]);
        }

        return (int)(vrednost * 0.25 * this.C(u) * this.C(v)); ;
    }
    #endregion

    #region Cik-cakanje
    private void cikcakanje(int numMatrika, int x, int y)
    {
        /*
        sekvenca[0] = this.matrikaRGB[numMatrika][x + 0, y + 0]; // 0,0
        sekvenca[1] = this.matrikaRGB[numMatrika][x + 0, y + 1]; // 0,1
        sekvenca[2] = this.matrikaRGB[numMatrika][x + 1, y + 0]; // 1,0
        sekvenca[3] = this.matrikaRGB[numMatrika][x + 2, y + 0]; // 2,0
        sekvenca[4] = this.matrikaRGB[numMatrika][x + 1, y + 1]; // 1,1
        sekvenca[5] = this.matrikaRGB[numMatrika][x + 0, y + 2]; // 0,2
        sekvenca[6] = this.matrikaRGB[numMatrika][x + 0, y + 3]; // 0,3
        sekvenca[7] = this.matrikaRGB[numMatrika][x + 1, y + 2]; // 1,2
        sekvenca[8] = this.matrikaRGB[numMatrika][x + 2, y + 1]; // 2,1
        sekvenca[9] = this.matrikaRGB[numMatrika][x + 3, y + 0]; // 3,0
        sekvenca[10] = this.matrikaRGB[numMatrika][x + 4, y + 0]; // 4,0
        sekvenca[11] = this.matrikaRGB[numMatrika][x + 3, y + 1]; // 3,1
        sekvenca[12] = this.matrikaRGB[numMatrika][x + 2, y + 2]; // 2,2
        sekvenca[13] = this.matrikaRGB[numMatrika][x + 1, y + 3]; // 1,3
        sekvenca[14] = this.matrikaRGB[numMatrika][x + 0, y + 4]; // 0,4
        sekvenca[15] = this.matrikaRGB[numMatrika][x + 0, y + 5]; // 0,5
        sekvenca[16] = this.matrikaRGB[numMatrika][x + 1, y + 4]; // 1,4
        sekvenca[17] = this.matrikaRGB[numMatrika][x + 2, y + 3]; // 2,3
        sekvenca[18] = this.matrikaRGB[numMatrika][x + 3, y + 2]; // 3,2
        sekvenca[19] = this.matrikaRGB[numMatrika][x + 4, y + 1]; // 4,1
        sekvenca[20] = this.matrikaRGB[numMatrika][x + 5, y + 0]; // 5,0
        sekvenca[21] = this.matrikaRGB[numMatrika][x + 6, y + 0]; // 6,0
        sekvenca[22] = this.matrikaRGB[numMatrika][x + 5, y + 1]; // 5,1
        sekvenca[23] = this.matrikaRGB[numMatrika][x + 4, y + 2]; // 4,2
        sekvenca[24] = this.matrikaRGB[numMatrika][x + 3, y + 3]; // 3,3
        sekvenca[25] = this.matrikaRGB[numMatrika][x + 2, y + 4]; // 2,4
        sekvenca[26] = this.matrikaRGB[numMatrika][x + 1, y + 5]; // 1,5
        sekvenca[27] = this.matrikaRGB[numMatrika][x + 0, y + 6]; // 0,6
        sekvenca[28] = this.matrikaRGB[numMatrika][x + 0, y + 7]; // 0,7
        sekvenca[29] = this.matrikaRGB[numMatrika][x + 1, y + 6]; // 1,6
        sekvenca[30] = this.matrikaRGB[numMatrika][x + 2, y + 5]; // 2,5
        sekvenca[31] = this.matrikaRGB[numMatrika][x + 3, y + 4]; // 3,4
        sekvenca[32] = this.matrikaRGB[numMatrika][x + 4, y + 3]; // 4,3
        sekvenca[33] = this.matrikaRGB[numMatrika][x + 5, y + 2]; // 5,2
        sekvenca[34] = this.matrikaRGB[numMatrika][x + 6, y + 1]; // 6,1
        sekvenca[35] = this.matrikaRGB[numMatrika][x + 7, y + 0]; // 7,0
        sekvenca[36] = this.matrikaRGB[numMatrika][x + 7, y + 1]; // 7,1
        sekvenca[37] = this.matrikaRGB[numMatrika][x + 6, y + 2]; // 6,2
        sekvenca[38] = this.matrikaRGB[numMatrika][x + 5, y + 3]; // 5,3
        sekvenca[39] = this.matrikaRGB[numMatrika][x + 4, y + 4]; // 4,4
        sekvenca[40] = this.matrikaRGB[numMatrika][x + 3, y + 5]; // 3,5
        sekvenca[41] = this.matrikaRGB[numMatrika][x + 2, y + 6]; // 2,6
        sekvenca[42] = this.matrikaRGB[numMatrika][x + 1, y + 7]; // 1,7
        sekvenca[43] = this.matrikaRGB[numMatrika][x + 2, y + 7]; // 2,7
        sekvenca[44] = this.matrikaRGB[numMatrika][x + 3, y + 6]; // 3,6
        sekvenca[45] = this.matrikaRGB[numMatrika][x + 4, y + 5]; // 4,5
        sekvenca[46] = this.matrikaRGB[numMatrika][x + 5, y + 4]; // 5,4
        sekvenca[47] = this.matrikaRGB[numMatrika][x + 6, y + 3]; // 6,3
        sekvenca[48] = this.matrikaRGB[numMatrika][x + 7, y + 2]; // 7,2
        sekvenca[49] = this.matrikaRGB[numMatrika][x + 7, y + 3]; // 7,3
        sekvenca[50] = this.matrikaRGB[numMatrika][x + 6, y + 4]; // 6,4
        sekvenca[51] = this.matrikaRGB[numMatrika][x + 5, y + 5]; // 5,5
        sekvenca[52] = this.matrikaRGB[numMatrika][x + 4, y + 6]; // 4,6
        sekvenca[53] = this.matrikaRGB[numMatrika][x + 3, y + 7]; // 3,7
        sekvenca[54] = this.matrikaRGB[numMatrika][x + 4, y + 7]; // 4,7
        sekvenca[55] = this.matrikaRGB[numMatrika][x + 5, y + 6]; // 5,6
        sekvenca[56] = this.matrikaRGB[numMatrika][x + 6, y + 5]; // 6,5
        sekvenca[57] = this.matrikaRGB[numMatrika][x + 7, y + 4]; // 7,4
        sekvenca[58] = this.matrikaRGB[numMatrika][x + 7, y + 5]; // 7,5
        sekvenca[59] = this.matrikaRGB[numMatrika][x + 6, y + 6]; // 6,6
        sekvenca[60] = this.matrikaRGB[numMatrika][x + 5, y + 7]; // 5,7
        sekvenca[61] = this.matrikaRGB[numMatrika][x + 6, y + 7]; // 6,7
        sekvenca[62] = this.matrikaRGB[numMatrika][x + 7, y + 6]; // 7,6
        sekvenca[63] = this.matrikaRGB[numMatrika][x + 7, y + 7]; // 7,7
        */

        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 0]); // 0,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 1]); // 0,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 0]); // 1,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 0]); // 2,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 1]); // 1,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 2]); // 0,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 3]); // 0,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 2]); // 1,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 1]); // 2,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 0]); // 3,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 0]); // 4,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 1]); // 3,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 2]); // 2,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 3]); // 1,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 4]); // 0,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 5]); // 0,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 4]); // 1,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 3]); // 2,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 2]); // 3,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 1]); // 4,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 0]); // 5,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 0]); // 6,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 1]); // 5,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 2]); // 4,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 3]); // 3,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 4]); // 2,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 5]); // 1,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 6]); // 0,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 0, y + 7]); // 0,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 6]); // 1,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 5]); // 2,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 4]); // 3,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 3]); // 4,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 2]); // 5,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 1]); // 6,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 0]); // 7,0
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 1]); // 7,1
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 2]); // 6,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 3]); // 5,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 4]); // 4,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 5]); // 3,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 6]); // 2,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 1, y + 7]); // 1,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 2, y + 7]); // 2,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 6]); // 3,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 5]); // 4,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 4]); // 5,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 3]); // 6,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 2]); // 7,2
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 3]); // 7,3
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 4]); // 6,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 5]); // 5,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 6]); // 4,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 3, y + 7]); // 3,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 4, y + 7]); // 4,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 6]); // 5,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 5]); // 6,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 4]); // 7,4
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 5]); // 7,5
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 6]); // 6,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 5, y + 7]); // 5,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 6, y + 7]); // 6,7
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 6]); // 7,6
        this.cikCak.Add(this.matrikaRGB[numMatrika][x + 7, y + 7]); // 7,7
    }
    #endregion

    #region Cik-Cak
    private void CikCak()
    {
        for (int numMatrika = 0; numMatrika < 3; numMatrika++)
        {
            for (int a = 0; a < numVrstica; a++)
            {
                for (int b = 0; b < numStolpec; b++)
                    cikcakanje(numMatrika, a * 8, b * 8);
            }
        }
    }
    #endregion

    #region Kodiranje - Faktor od 0 do 14
    private void KodiranjeDo14()
    {
        for (int i = 0; i < this.cikCak.Count; i++)
        {
            this.kodirniNiz = new List<bool>();
            if (this.cikCak[i] != 0) // opcija C
            {
                this.dec2bin(this.cikCak[i]); // pretvorimo v binarno vrednost in zapišemo kodirni niz
            }
            else // opcija A ali B
            {
                this.numNicel = 1;

                // i + 1 je manjši od dolžine polja, naslednji člen je enak 0 in če naslednji člen ni konec 8x8 matrike, ponovi zanko
                while (((i + 1) != this.cikCak.Count) && (this.cikCak[i + 1] == 0) && (((i + 1) % 64) != 0))
                {
                    this.numNicel++;
                    i++;
                }

                this.nicle2bin(this.numNicel); // število ničel, zapišemo z 6 biti

                if ((i + 1) % 64 != 0) // opcija A, saj nismo prišli do konca (8x8)
                {
                    i++;
                    this.dec2bin(this.cikCak[i]);
                }
                else // opcija B
                    ShraniKodirniNiz();
            }
        }
    }
    #endregion

    #region Kodiranje - Faktor 14
    private void Kodiranje14()
    {
        for (int i = 0; i < this.cikCak.Count; i = i + 64)
        {
            this.kodirniNiz = new List<bool>();
            this.dec2bin(this.cikCak[i]); // pretvorimo v binarno vrednost in zapišemo kodirni niz
        }
    }
    #endregion

    #region Kodiranje - Faktor 15
    private void Kodiranje15()
    {
        int stevilo = this.cikCak.Count;

        bool[] bajtek = new bool[32];

        for (int i = 0; i < 32; i++)
            bajtek[i] = false;

        int indeks = 0;

        // pretvorba
        do
        {
            bajtek[indeks++] = Convert.ToBoolean(stevilo % 2);
            //zacasnoPolje.Add(Convert.ToBoolean(stevilo % 2));
            stevilo = stevilo >> 1;
        } while (stevilo > 0);

        // obrnemo polje
        indeks = 0;
        bool[] obrnjeno = new bool[32];
        for (int i = 31; i >= 0; i--)
            obrnjeno[indeks++] = bajtek[i];

        // dejansko število zapisano z biti
        for (int i = 0; i < 32; i++)
            this.datoteka.Add(obrnjeno[i]);
    }
    #endregion

    #region Shranjevanje
    private void Shranjevanje()
    {
        int bytes = this.datoteka.Count / 8;
        if ((this.datoteka.Count % 8) != 0) bytes++;

        // +5 -> shranimo širino in višino slike na začetek datoteke + 1 za faktor
        byte[] array = new byte[bytes + 5];
        array[0] = Convert.ToByte((uint)(this.slika.Width << 16) >> 24);
        array[1] = Convert.ToByte((uint)(this.slika.Width << 24) >> 24);
        array[2] = Convert.ToByte((uint)(this.slika.Height << 16) >> 24);
        array[3] = Convert.ToByte((uint)(this.slika.Height << 24) >> 24);

        array[4] = Convert.ToByte((uint)(this.faktorStiskanja)); // shranimo faktor stiskanja

        int bitIndex = 0;
        int byteIndex = 5; // ni 0, če mi dodamo 5 bytov

        for (int i = 4; i < this.datoteka.Count + 4; i++)
        {
            if (this.datoteka[i - 4])
                array[byteIndex] |= (byte)(((byte)1) << bitIndex);

            bitIndex++;

            if (bitIndex == 8)
            {
                bitIndex = 0;
                byteIndex++;
            }
        }

        // Shranimo celotno polje bajtov
        System.IO.FileStream shranjevanje = new System.IO.FileStream(this.potDoDatoteke, System.IO.FileMode.Create, System.IO.FileAccess.Write);
        shranjevanje.Write(array, 0, array.Length);
        shranjevanje.Close();

        #region Testni izpis
        /*
        Console.WriteLine("SHRANJEVANJE");
        for (int i = 0; i < 20; i++)
            Console.WriteLine("i: " + i + "\t this.datoteka: " + this.datoteka[i]);
        for (int i = 76115; i < this.datoteka.Count; i++)
            Console.WriteLine("i: " + i + "\t this.datoteka: " + this.datoteka[i]);
        */
        #endregion
    }
    #endregion

    // * Branje -> Dekodiranje -> Cik-Cak -> IDCT *

    #region Branje
    private void Branje()
    {
        byte[] buffer = null;

        // skopirano iz interneta, popravljeno le imena spremenljivk
        System.IO.FileStream branje = new System.IO.FileStream(this.potDoDatoteke, System.IO.FileMode.Open, System.IO.FileAccess.Read);
        System.IO.BinaryReader binaryReader = new System.IO.BinaryReader(branje);
        long velikost = new System.IO.FileInfo(this.potDoDatoteke).Length;
        buffer = binaryReader.ReadBytes((Int32)velikost);

        branje.Close();
        branje.Dispose();
        binaryReader.Close();

        this.numVrstica = 0;
        this.numStolpec = 0;

        for (int i = 0; i < 2; i++)
            this.numVrstica += Convert.ToInt32(buffer[i]) << ((1 - i) * 8);

        for (int i = 2; i < 4; i++)
            this.numStolpec += Convert.ToInt32(buffer[i]) << ((3 - i) * 8);

        this.faktorStiskanja = Convert.ToInt32(buffer[4]); // dobimo faktor stiskanja

        #region * vrstice in stolpci *
        this.numVrstica = this.slika.Width / 8;
        this.numStolpec = this.slika.Height / 8;

        if (this.slika.Width % 8 != 0)
            this.numVrstica++;
        if (this.slika.Height % 8 != 0)
            this.numStolpec++;
        #endregion

        this.datoteka.Clear();

        bool[] bajtek = new bool[8];
        for (int i = 5; i < buffer.Length; i++)
        {
            this.tmpVelikost = 0;
            bajtek = this.DodajDolzino(bajtek, 8, (int)buffer[i]);
            this.tmpVelikost = 0;

            do
            {
                this.datoteka.Add(bajtek[this.tmpVelikost++]);
            } while (this.tmpVelikost < 8);
        }

        this.velikostKompresija = this.datoteka.Count;

        #region Testni izpis
        /*
        Console.WriteLine("\n\nBRANJE");
        for (int i = 0; i < 20; i++)
            Console.WriteLine("i: " + i + "\t this.datoteka: " + this.datoteka[i]);
        for (int i = 76115; i < this.datoteka.Count; i++)
            Console.WriteLine("i: " + i + "\t this.datoteka: " + this.datoteka[i]);
        */
        #endregion
    }
    #endregion

    #region Dekodiranje - Faktor od 0 do 14
    private void DekodiranjeDo14()
    {
        this.cikCak.Clear();

        for (int i = 0; i < this.datoteka.Count; i++)
        {
            if (this.datoteka[i] != false) // število
                i = this.bin2dec(i);
            else // ničla
                i = this.dodajNicle(i);
        }
    }
    #endregion

    #region Dekodiranje - Faktor 14
    private void Dekodiranje14()
    {
        this.cikCak.Clear();

        for (int i = 0; i < this.datoteka.Count; i++)
        {
            if (i + 1 < this.datoteka.Count)
            {
                i = this.bin2dec(i);

                for (int j = 0; j < 63; j++)
                    this.cikCak.Add(0);
            }
        }
    }
    #endregion

    #region Dekodiranje - Faktor 15
    private void Dekodiranje15()
    {
        this.cikCak.Clear();

        int stevilo = 0;

        for (int i = 0; i < this.datoteka.Count; i++)
        {
            stevilo = stevilo << 1;
            stevilo += Convert.ToInt32(this.datoteka[i]);
        }

        for (int i = 0; i < stevilo; i++)
            this.cikCak.Add(0);
    }
    #endregion

    #region Re-Cik-cakanje
    private void odcikcakanje(int numMatrika, int x, int y)
    {
        // 0-63 (x=0, y=0)
        // 64-127 (x=0, y=8)
        // 128-191 (x=0, y=16)
        /*
        if (seq + 64 >= this.cikCak.Count)
            return;
        */
        this.matrikaRGB[numMatrika][x + 0, y + 0] = this.cikCak[seq++]; // 0,0
        this.matrikaRGB[numMatrika][x + 0, y + 1] = this.cikCak[seq++]; // 0,1
        this.matrikaRGB[numMatrika][x + 1, y + 0] = this.cikCak[seq++]; // 1,0
        this.matrikaRGB[numMatrika][x + 2, y + 0] = this.cikCak[seq++]; // 2,0
        this.matrikaRGB[numMatrika][x + 1, y + 1] = this.cikCak[seq++]; // 1,1
        this.matrikaRGB[numMatrika][x + 0, y + 2] = this.cikCak[seq++]; // 0,2
        this.matrikaRGB[numMatrika][x + 0, y + 3] = this.cikCak[seq++]; // 0,3
        this.matrikaRGB[numMatrika][x + 1, y + 2] = this.cikCak[seq++]; // 1,2
        this.matrikaRGB[numMatrika][x + 2, y + 1] = this.cikCak[seq++]; // 2,1
        this.matrikaRGB[numMatrika][x + 3, y + 0] = this.cikCak[seq++]; // 3,0
        this.matrikaRGB[numMatrika][x + 4, y + 0] = this.cikCak[seq++]; // 4,0
        this.matrikaRGB[numMatrika][x + 3, y + 1] = this.cikCak[seq++]; // 3,1
        this.matrikaRGB[numMatrika][x + 2, y + 2] = this.cikCak[seq++]; // 2,2
        this.matrikaRGB[numMatrika][x + 1, y + 3] = this.cikCak[seq++]; // 1,3
        this.matrikaRGB[numMatrika][x + 0, y + 4] = this.cikCak[seq++]; // 0,4
        this.matrikaRGB[numMatrika][x + 0, y + 5] = this.cikCak[seq++]; // 0,5
        this.matrikaRGB[numMatrika][x + 1, y + 4] = this.cikCak[seq++]; // 1,4
        this.matrikaRGB[numMatrika][x + 2, y + 3] = this.cikCak[seq++]; // 2,3
        this.matrikaRGB[numMatrika][x + 3, y + 2] = this.cikCak[seq++]; // 3,2
        this.matrikaRGB[numMatrika][x + 4, y + 1] = this.cikCak[seq++]; // 4,1
        this.matrikaRGB[numMatrika][x + 5, y + 0] = this.cikCak[seq++]; // 5,0
        this.matrikaRGB[numMatrika][x + 6, y + 0] = this.cikCak[seq++]; // 6,0
        this.matrikaRGB[numMatrika][x + 5, y + 1] = this.cikCak[seq++]; // 5,1
        this.matrikaRGB[numMatrika][x + 4, y + 2] = this.cikCak[seq++]; // 4,2
        this.matrikaRGB[numMatrika][x + 3, y + 3] = this.cikCak[seq++]; // 3,3
        this.matrikaRGB[numMatrika][x + 2, y + 4] = this.cikCak[seq++]; // 2,4
        this.matrikaRGB[numMatrika][x + 1, y + 5] = this.cikCak[seq++]; // 1,5
        this.matrikaRGB[numMatrika][x + 0, y + 6] = this.cikCak[seq++]; // 0,6
        this.matrikaRGB[numMatrika][x + 0, y + 7] = this.cikCak[seq++]; // 0,7
        this.matrikaRGB[numMatrika][x + 1, y + 6] = this.cikCak[seq++]; // 1,6
        this.matrikaRGB[numMatrika][x + 2, y + 5] = this.cikCak[seq++]; // 2,5
        this.matrikaRGB[numMatrika][x + 3, y + 4] = this.cikCak[seq++]; // 3,4
        this.matrikaRGB[numMatrika][x + 4, y + 3] = this.cikCak[seq++]; // 4,3
        this.matrikaRGB[numMatrika][x + 5, y + 2] = this.cikCak[seq++]; // 5,2
        this.matrikaRGB[numMatrika][x + 6, y + 1] = this.cikCak[seq++]; // 6,1
        this.matrikaRGB[numMatrika][x + 7, y + 0] = this.cikCak[seq++]; // 7,0
        this.matrikaRGB[numMatrika][x + 7, y + 1] = this.cikCak[seq++]; // 7,1
        this.matrikaRGB[numMatrika][x + 6, y + 2] = this.cikCak[seq++]; // 6,2
        this.matrikaRGB[numMatrika][x + 5, y + 3] = this.cikCak[seq++]; // 5,3
        this.matrikaRGB[numMatrika][x + 4, y + 4] = this.cikCak[seq++]; // 4,4
        this.matrikaRGB[numMatrika][x + 3, y + 5] = this.cikCak[seq++]; // 3,5
        this.matrikaRGB[numMatrika][x + 2, y + 6] = this.cikCak[seq++]; // 2,6
        this.matrikaRGB[numMatrika][x + 1, y + 7] = this.cikCak[seq++]; // 1,7
        this.matrikaRGB[numMatrika][x + 2, y + 7] = this.cikCak[seq++]; // 2,7
        this.matrikaRGB[numMatrika][x + 3, y + 6] = this.cikCak[seq++]; // 3,6
        this.matrikaRGB[numMatrika][x + 4, y + 5] = this.cikCak[seq++]; // 4,5
        this.matrikaRGB[numMatrika][x + 5, y + 4] = this.cikCak[seq++]; // 5,4
        this.matrikaRGB[numMatrika][x + 6, y + 3] = this.cikCak[seq++]; // 6,3
        this.matrikaRGB[numMatrika][x + 7, y + 2] = this.cikCak[seq++]; // 7,2
        this.matrikaRGB[numMatrika][x + 7, y + 3] = this.cikCak[seq++]; // 7,3
        this.matrikaRGB[numMatrika][x + 6, y + 4] = this.cikCak[seq++]; // 6,4
        this.matrikaRGB[numMatrika][x + 5, y + 5] = this.cikCak[seq++]; // 5,5
        this.matrikaRGB[numMatrika][x + 4, y + 6] = this.cikCak[seq++]; // 4,6
        this.matrikaRGB[numMatrika][x + 3, y + 7] = this.cikCak[seq++]; // 3,7
        this.matrikaRGB[numMatrika][x + 4, y + 7] = this.cikCak[seq++]; // 4,7
        this.matrikaRGB[numMatrika][x + 5, y + 6] = this.cikCak[seq++]; // 5,6
        this.matrikaRGB[numMatrika][x + 6, y + 5] = this.cikCak[seq++]; // 6,5
        this.matrikaRGB[numMatrika][x + 7, y + 4] = this.cikCak[seq++]; // 7,4
        this.matrikaRGB[numMatrika][x + 7, y + 5] = this.cikCak[seq++]; // 7,5
        this.matrikaRGB[numMatrika][x + 6, y + 6] = this.cikCak[seq++]; // 6,6
        this.matrikaRGB[numMatrika][x + 5, y + 7] = this.cikCak[seq++]; // 5,7
        this.matrikaRGB[numMatrika][x + 6, y + 7] = this.cikCak[seq++]; // 6,7
        this.matrikaRGB[numMatrika][x + 7, y + 6] = this.cikCak[seq++]; // 7,6
        this.matrikaRGB[numMatrika][x + 7, y + 7] = this.cikCak[seq++]; // 7,7
    }
    #endregion

    #region Cik-Cak
    private void ReCikCak()
    {
        for (int numMatrika = 0; numMatrika < 3; numMatrika++)
        {
            for (int a = 0; a < this.numVrstica; a++)
            {
                for (int b = 0; b < this.numStolpec; b++)
                    odcikcakanje(numMatrika, a * 8, b * 8);
            }
        }
    }
    #endregion

    #region Algoritem IDCT
    private void AlgoritemIDCT()
    {
        Parallel.For(0, 3, delegate(int numMatrika)
        {
            for (int a = 0; a < this.numVrstica; a++)
            {
                for (int b = 0; b < this.numStolpec; b++)
                {
                    for (int x = 0; x < enBajt; x++)
                    {
                        for (int y = 0; y < enBajt; y++)
                            this.matrikaIDCT[numMatrika][x + (a * enBajt), y + (b * enBajt)] = this.IDCT(x, y, a, b, this.matrikaRGB[numMatrika]);
                    }
                }
            }
        });
    }

    private double temp = (1 / Math.Sqrt(2));

    private int IDCT(int x, int y, int numX, int numY, int[,] matrika)
    {
        numX = numX * 8;
        numY = numY * 8;

        double vrednost = 0;

        for (int v = 0; v < enBajt; v++)
        {
            for (int u = 0; u < enBajt; u++)
            {
                if ((u == 0 && v != 0) || (u != 0 && v == 0))
                    vrednost += this.temp * (double)(matrika[v + numX, u + numY]) * this.cos[x, u] * this.cos[y, v];
                else if (u != 0 && v != 0)
                    vrednost += (double)(matrika[v + numX, u + numY]) * this.cos[x, u] * this.cos[y, v];
                else
                    vrednost += 0.5 * (double)(matrika[v + numX, u + numY]) * this.cos[x, u] * this.cos[y, v];

                /*
                if (u == 0 || v == 0)
                    vrednost += this.C(u) * this.C(v) * (double)(matrika[v + numX, u + numY]) * this.cos[x, u] * this.cos[y, v];
                else
                    vrednost += (double)(matrika[v + numX, u + numY]) * this.cos[x, u] * this.cos[y, v];
                */
            }
        }
        vrednost = Math.Round((vrednost * 0.25) + 128);

        return (vrednost > 255) ? 255 : (vrednost < 0) ? 0 : (int)vrednost;
    }
    #endregion

    // * Pomožne funkcije *

    #region Pomožna funkcija - DodajDolzino (Branje)
    /*
        Pomožna funkcija - DodajDolzino služi za dodajanje dolžine v bitih v polje z že obstoječo vrednostjo v bitih
        Parametri funkcije:
        - poljeBit: polje bitov
        - velikost: velikost števila dolžine v bitih
        - dolzina: dolžina, ki jo želimo zapisati
        Funkcija vrne polje bitov, kjer je shranjena pretvorjena vrednost
    */
    private bool[] DodajDolzino(bool[] poljeBit, int velikost, int dolzina)
    {
        for (int j = 0; j < velikost; j++)
        {
            poljeBit[this.tmpVelikost] = Convert.ToBoolean(dolzina % 2);
            dolzina = dolzina >> 1;
            this.tmpVelikost++;
        }

        return poljeBit;
    }
    #endregion

    #region Pomožna funkcija - ShraniKodirniNiz()
    private void ShraniKodirniNiz()
    {
        // kodirni niz sestavljen, vpis v eno spremenljivko (primerno za shraniti v datoteko)
        for (int i = 0; i < this.kodirniNiz.Count; i++)
            this.datoteka.Add(this.kodirniNiz[i]);
    }
    #endregion

    #region Pomožna funkcija - nicle2bin
    private void nicle2bin(int stevilo)
    {
        this.kodirniNiz.Add(false); // 0 -> ničle (vodilni bit)
        List<bool> zacasnoPolje = new List<bool>();

        do
        {
            zacasnoPolje.Add(Convert.ToBoolean(stevilo % 2));
            stevilo = stevilo >> 1;
        } while (stevilo > 0);

        while (zacasnoPolje.Count < 6)
            zacasnoPolje.Add(false);

        // obrnemo polje
        zacasnoPolje.Reverse();

        for (int j = 0; j < zacasnoPolje.Count; j++)
            this.kodirniNiz.Add(zacasnoPolje[j]);
    }
    #endregion

    #region Kodiranje: dec 2 bin
    // funkcija, ki vrne tekočo dolžino
    private bool[] vrniDolzino(int dolzina)
    {
        bool[] polje = new bool[4];

        switch (dolzina)
        {
            case 0: polje[0] = false; polje[1] = false; polje[2] = false; polje[3] = false; break;
            case 1: polje[0] = false; polje[1] = false; polje[2] = false; polje[3] = true; break;
            case 2: polje[0] = false; polje[1] = false; polje[2] = true; polje[3] = false; break;
            case 3: polje[0] = false; polje[1] = false; polje[2] = true; polje[3] = true; break;
            case 4: polje[0] = false; polje[1] = true; polje[2] = false; polje[3] = false; break;
            case 5: polje[0] = false; polje[1] = true; polje[2] = false; polje[3] = true; break;
            case 6: polje[0] = false; polje[1] = true; polje[2] = true; polje[3] = false; break;
            case 7: polje[0] = false; polje[1] = true; polje[2] = true; polje[3] = true; break;
            case 8: polje[0] = true; polje[1] = false; polje[2] = false; polje[3] = false; break;
            case 9: polje[0] = true; polje[1] = false; polje[2] = false; polje[3] = true; break;
            case 10: polje[0] = true; polje[1] = false; polje[2] = true; polje[3] = false; break;
            case 11: polje[0] = true; polje[1] = false; polje[2] = true; polje[3] = true; break;
            case 12: polje[0] = true; polje[1] = true; polje[2] = false; polje[3] = false; break;
            case 13: polje[0] = true; polje[1] = true; polje[2] = false; polje[3] = true; break;
            case 14: polje[0] = true; polje[1] = true; polje[2] = true; polje[3] = false; break;
            case 15: polje[0] = true; polje[1] = true; polje[2] = true; polje[3] = true; break;
        }

        return polje;
    }

    // funkcija sestavi kodirni niz
    private void dec2bin(int stevilo)
    {
        // negativno število?
        bool negativnoStevilo = false;
        if (stevilo < 0)
        {
            negativnoStevilo = true;
            stevilo = stevilo * (-1);
        }

        List<bool> zacasnoPolje = new List<bool>();

        // pretvorba
        do
        {
            zacasnoPolje.Add(Convert.ToBoolean(stevilo % 2));
            stevilo = stevilo >> 1;
        } while (stevilo > 0);

        // obrnemo polje
        zacasnoPolje.Reverse();

        // zapišemo z 4 biti, koliko bitov sledi (število)
        bool[] polje = vrniDolzino(zacasnoPolje.Count);

        this.kodirniNiz.Add(true); // 1 -> število (vodilni bit)
        this.kodirniNiz.Add(negativnoStevilo); // negativno = true (-), dodamo true, drugače false (+)
        // dolžina 4 biti
        for (int i = 0; i < 4; i++)
            this.kodirniNiz.Add(polje[i]);
        // dejansko število zapisano z biti
        for (int i = 0; i < zacasnoPolje.Count; i++)
            this.kodirniNiz.Add(zacasnoPolje[i]);

        // kodirni niz sestavljen, vpis v eno spremenljivko (primerno za shraniti v datoteko)
        ShraniKodirniNiz();
    }
    #endregion

    #region Dekodiranje: bin 2 dec
    private int bin2dec(int trenutniIndex)
    {
        bool negativno = this.datoteka[++trenutniIndex];

        List<bool> polje = new List<bool>();
        for (int i = 0; i < 4; i++)
        {
            if (trenutniIndex + 1 < this.datoteka.Count)
                polje.Add(this.datoteka[++trenutniIndex]);
        }

        int dolzina = 0;
        for (int i = 0; i < polje.Count; i++)
        {
            dolzina = dolzina << 1;
            dolzina += Convert.ToInt32(polje[i]);
        }

        polje.Clear();
        for (int i = 0; i < dolzina; i++)
        {
            if (trenutniIndex + 1 < this.datoteka.Count)
                polje.Add(this.datoteka[++trenutniIndex]);
        }

        int stevilo = 0;
        for (int i = 0; i < polje.Count; i++)
        {
            stevilo = stevilo << 1;
            stevilo += Convert.ToInt32(polje[i]);
        }

        if (negativno)
            stevilo = stevilo * (-1);

        this.cikCak.Add(stevilo);

        return trenutniIndex;
    }

    private int dodajNicle(int trenutniIndex)
    {
        List<bool> polje = new List<bool>();
        for (int i = 0; i < 6; i++)
        {
            if (trenutniIndex + 1 < this.datoteka.Count)
                polje.Add(this.datoteka[++trenutniIndex]);
        }

        this.numNicel = 0;
        for (int i = 0; i < polje.Count; i++)
        {
            this.numNicel = this.numNicel << 1;
            this.numNicel += Convert.ToInt32(polje[i]);
        }

        for (int i = 0; i < this.numNicel; i++)
            this.cikCak.Add(0);

        return trenutniIndex;
    }
    #endregion

    // * Izpis časov *

    #region Izpis časov - Prva stopnja
    void IzpisPrvaStopnja()
    {
        this.casBranjeRGB = shranjeniCas[0];
        this.casFDCT = shranjeniCas[1];
        this.casIDCT = shranjeniCas[2];
        this.casIzris = shranjeniCas[3];
        this.skupniCas = shranjeniCas[4];
    }
    #endregion

    #region Izpis časov - Druga stopnja
    void IzpisDrugaStopnja()
    {
        this.casBranjeRGB = shranjeniCas[0];
        this.casFDCT = shranjeniCas[1];
        this.casCikCak = shranjeniCas[2];
        this.casKodiranje = shranjeniCas[3];
        this.casShranjevanje = shranjeniCas[4];
        this.casBranje = shranjeniCas[5];
        this.casDekodiranje = shranjeniCas[6];
        this.casReCikCak = shranjeniCas[7];
        this.casIDCT = shranjeniCas[8];
        this.casIzris = shranjeniCas[9];
        this.skupniCas = shranjeniCas[10];
    }
    #endregion
}
