﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Resources;
using System.Windows.Media.Imaging;
using Microsoft.Phone;
using System.Collections.ObjectModel;
using Microsoft.Xna.Framework.Media;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Data;
using System.Threading;
using System.Diagnostics;

public class RazpoznavanjeSlike
{
    // * Spremenljivke *

    #region * Spremenljivke *

    #region * razredne spremenljivke - public *
    public WriteableBitmap slika; // objekt slika
    #endregion

    #region * razredne spremenljivke - private *
    private List<List<double>> matrika = new List<List<double>>(); // najprej si bomo shranili vrednosti ((R+G+B) / 3) = sivina
    private List<double> tmpRGB = new List<double>();
    List<Point> seznamTock = new List<Point>();
    List<Point> seznamSredisc = new List<Point>();
    private Color barva;
    private int stCrnih = 0;
    #endregion

    #endregion

    // * Funkciji za branje in risanje slike *

    #region Branje slike (RGB)
    private void BranjeSlike()
    {
        double x = 0;

        for (int i = 0; i < this.slika.PixelWidth; i++)
        {
            this.tmpRGB = new List<double>();

            for (int j = 0; j < this.slika.PixelHeight; j++)
            {
                this.barva = this.slika.GetPixel(i, j);

                x = 0.299 * this.barva.R + 0.587 * this.barva.G + 0.114 * this.barva.B;

                this.tmpRGB.Add(x);
            }

            this.matrika.Add(this.tmpRGB);
        }
    }
    #endregion

    // * Konstruktor *

    #region * Konstruktor *
    public RazpoznavanjeSlike(WriteableBitmap slika)
    {
        // "inicializacija" razrednih spremenljivk "public" (prireditev)
        this.slika = new WriteableBitmap(slika);
        //this.sivinskaSlika = new WriteableBitmap(slika);

        this.matrika.Clear();
        this.seznamSredisc.Clear();

        this.BranjeSlike();
        this.PretvorbaCrnoBela();
        this.Razpoznavanje();
    }
    #endregion

    // * Funkcije (črno-bela slika, razpoznavanje) *

    #region Pretvorba v črno belo sliko (če je vrednost manjša od povprečja = črna, drugače = bela)
    private void PretvorbaCrnoBela()
    {
        for (int i = 0; i < this.slika.PixelWidth; i++)
        {
            for (int j = 0; j < this.slika.PixelHeight; j++)
            {
                if ((this.matrika[i][j] >= 0) && (this.matrika[i][j] <= 90))
                {
                    this.matrika[i][j] = 0;
                    stCrnih++;
                }
                else
                    this.matrika[i][j] = 255;
            }
        }
    }
    #endregion

    #region Razpoznavanje (dobimo središča)
    public bool Razpoznavanje()
    {
        double procentCrnih = (double)stCrnih / (double)(this.slika.PixelWidth * this.slika.PixelHeight);

        if ((procentCrnih < 0.07) && (procentCrnih > 0.002))
        {
            // začneš, ko najdeš prvi črn pixel, ideš rekurzivno in si shranjuješ črne in na koncu vrneš X,Y, potem pa greš iskat naslednjega črnega, kar bo pomenilo nov krogec
            this.seznamTock.Clear();

            for (int i = 0; i < this.slika.PixelHeight; i++)
            {
                for (int j = 0; j < this.slika.PixelWidth; j++)
                {
                    // najden črn pixel
                    if (this.matrika[j][i] == 0)
                    {
                        this.PoisciTocke(j, i);
                        this.seznamSredisc.Add(this.VrniSredisce());
                    }
                    this.seznamTock.Clear();
                }
            }

            return true;
        }
        else
            return false;
    }
    #endregion

    #region Iščemo točke, tako da dobimo "krog"
    private void PoisciTocke(int x, int y)
    {
        bool tockaObstaja = false;

        for (int i = 0; i < this.seznamTock.Count; i++)
        {
            if ((this.seznamTock[i].X == x) && (this.seznamTock[i].Y == y))
                tockaObstaja = true;
        }

        this.matrika[x][y] = 255;

        if (!tockaObstaja)
            this.seznamTock.Add(new Point(x, y));

        /*
            (x-1),(y+1) |   (x),(y+1) |     (x+1)(y+1)
            -------------------------------------------
            (x-1),(y)   |   (x),(y)   |     (x+1),(y)
            -------------------------------------------
            (x-1),(y-1) |   (x),(y-1) |     (x+1),(y-1)
        */
        /*
        for (int k = 1; k <= 3; k++)
        {
            if ((y + k) < this.slika.PixelHeight)
            {
                if (this.matrika[x - k][y + k] == 0) // levo - zgoraj
                    this.PoisciTocke((x - k), (y + k));
            }

            if (this.matrika[x - k][y] == 0) // levo - sredina
                this.PoisciTocke((x - k), y);

            if (this.matrika[x - k][y - k] == 0) // levo - spodaj
                this.PoisciTocke((x - k), (y - k));

            if ((y + k) < this.slika.PixelHeight)
            {
                if (this.matrika[x][y + k] == 0) // sredina - zgoraj
                    this.PoisciTocke(x, (y + k));
            }

            if (this.matrika[x][y - k] == 0) // sredina - spodaj
                this.PoisciTocke(x, (y - k));

            if (((y + k) < this.slika.PixelHeight) && (x + k) < this.slika.PixelWidth)
            {
                if (this.matrika[x + k][y + k] == 0) // desno - zgoraj
                    this.PoisciTocke((x + k), (y + k));
            }

            if ((x + k) < this.slika.PixelWidth)
            {
                if (this.matrika[x + k][y] == 0) // desno - sredina
                    this.PoisciTocke((x + k), y);

                if (this.matrika[x + k][y - k] == 0) // desno - spodaj
                    this.PoisciTocke((x + k), (y - k));
            }
        }
        */
        for (int k = 1; k <= 3; k++)
        {
            if (((y + k) < this.slika.PixelHeight) && (x - k >= 0))
            {
                if (this.matrika[x - k][y + k] == 0) // levo - zgoraj
                    this.PoisciTocke((x - k), (y + k));
            }

            if (x - k >= 0)
            {
                if (this.matrika[x - k][y] == 0) // levo - sredina
                    this.PoisciTocke((x - k), y);
            }

            if ((x - k >= 0) && (y - k >= 0))
            {
                if (this.matrika[x - k][y - k] == 0) // levo - spodaj
                    this.PoisciTocke((x - k), (y - k));
            }

            if ((y + k) < this.slika.PixelHeight)
            {
                if (this.matrika[x][y + k] == 0) // sredina - zgoraj
                    this.PoisciTocke(x, (y + k));
            }

            if (y - k >= 0)
            {
                if (this.matrika[x][y - k] == 0) // sredina - spodaj
                    this.PoisciTocke(x, (y - k));
            }

            if (((y + k) < this.slika.PixelHeight) && (x + k) < this.slika.PixelWidth)
            {
                if (this.matrika[x + k][y + k] == 0) // desno - zgoraj
                    this.PoisciTocke((x + k), (y + k));
            }

            if (((x + k) < this.slika.PixelWidth) && (y - k >= 0))
            {
                if (this.matrika[x + k][y] == 0) // desno - sredina
                    this.PoisciTocke((x + k), y);

                if (this.matrika[x + k][y - k] == 0) // desno - spodaj
                    this.PoisciTocke((x + k), (y - k));
            }
        }
    }
    #endregion

    #region Določimo središče (X, Y)
    private Point VrniSredisce()
    {
        int minX = (int)this.seznamTock[0].X;
        int maxX = (int)this.seznamTock[0].X;
        int minY = (int)this.seznamTock[0].Y;
        int maxY = (int)this.seznamTock[0].Y;

        for (int i = 1; i < this.seznamTock.Count; i++)
        {
            if ((int)this.seznamTock[i].X < minX)
                minX = (int)this.seznamTock[i].X;

            if ((int)this.seznamTock[i].X > maxX)
                maxX = (int)this.seznamTock[i].X;

            if ((int)this.seznamTock[i].Y < minY)
                minY = (int)this.seznamTock[i].Y;

            if ((int)this.seznamTock[i].Y > maxY)
                maxY = (int)this.seznamTock[i].Y;
        }

        for (int i = minX; i <= maxX; i++)
        {
            for (int j = minY; j <= maxY; j++)
                matrika[i][j] = 255;
        }

        int x = (minX + maxX) / 2;
        int y = (minY + maxY) / 2;

        return new Point(x, y);
    }
    #endregion

    // * Vrnjen LIST *
    #region Vrne seznam središč
    public List<Point> DobiSeznamSredisc()
    {
        return this.seznamSredisc;
    }
    #endregion
}